<?php

/**
 * Class ilFormATestSkillEvaluationGUI
 * @ilCtrl_Calls ilFormATestSkillEvaluationGUI: ilTestSkillEvaluationToolbarGUI
 * @ilCtrl_Calls ilFormATestSkillEvaluationGUI: ilTestPersonalSkillsGUI
 * @see ilTestSkillEvaluationGUI
 */
class ilFormATestSkillEvaluationGUI
{
    const CMD_SHOW = 'show';

    /**
     * @var ilCtrl
     */
    private $ctrl;

    /**
     * @var ilTabsGUI
     */
    private $tabs;

    /**
     * @var ilGlobalTemplateInterface
     */
    private $tpl;

    /**
     * @var ilLanguage
     */
    private $lng;

    /**
     * @var ilDBInterface
     */
    private $db;

    /**
     * @var ilObjFormATest
     */
    protected $testOBJ;

    /**
     * @var ilTestSession
     */
    private $testSession;

    /**
     * @var array
     */
    private $testResults = [];

    /**
     * @var ilAssQuestionList
     */
    private $questionList;

    /**
     * @var ilTestObjectiveOrientedContainer
     */
    private $objectiveOrientedContainer;

    /**
     * @var bool
     */
    private $noSkillProfileOptionEnabled = false;

    /**
     * @var array
     */
    private $availableSkillProfiles = array();

    /**
     * @var array
     */
    private $availableSkills = array();

    /**
     * ilFormATestSkillEvaluationGUI constructor.
     * @param ilCtrl $ctrl
     * @param ilTabsGUI $tabs
     * @param ilGlobalTemplateInterface $tpl
     * @param ilLanguage $lng
     * @param ilDBInterface $db
     * @param ilObjFormATest $testOBJ
     */
    public function __construct(
        ilCtrl $ctrl,
        ilTabsGUI $tabs,
        ilGlobalTemplateInterface $tpl,
        ilLanguage $lng,
        ilDBInterface $db,
        ilObjFormATest $testOBJ
    ) {
        $this->ctrl = $ctrl;
        $this->tabs = $tabs;
        $this->tpl = $tpl;
        $this->lng = $lng;
        $this->db = $db;
        $this->testOBJ = $testOBJ;
    }

    public function executeCommand() : void
    {
        $cmd = $this->ctrl->getCmd(self::CMD_SHOW) . 'Cmd';

        $this->$cmd();
    }

    /**
     * @param array $testResults
     */
    public function setTestResults(array $testResults)
    {
        $this->testResults = $testResults;
    }

    /**
     * @return array
     */
    public function getTestResults() : array
    {
        return $this->testResults;
    }

    /**
     * @param ilTestSession $testSession
     */
    public function setTestSession($testSession)
    {
        $this->testSession = $testSession;
    }

    /**
     * @return ilAssQuestionList
     */
    public function getQuestionList()
    {
        return $this->questionList;
    }

    /**
     * @param ilAssQuestionList $questionList
     */
    public function setQuestionList($questionList)
    {
        $this->questionList = $questionList;
    }

    /**
     * @return ilTestObjectiveOrientedContainer
     */
    public function getObjectiveOrientedContainer()
    {
        return $this->objectiveOrientedContainer;
    }

    /**
     * @param ilTestObjectiveOrientedContainer $objectiveOrientedContainer
     */
    public function setObjectiveOrientedContainer($objectiveOrientedContainer)
    {
        $this->objectiveOrientedContainer = $objectiveOrientedContainer;
    }

    /**
     * @return boolean
     */
    public function isNoSkillProfileOptionEnabled()
    {
        return $this->noSkillProfileOptionEnabled;
    }

    /**
     * @param boolean $noSkillProfileOptionEnabled
     */
    public function setNoSkillProfileOptionEnabled($noSkillProfileOptionEnabled)
    {
        $this->noSkillProfileOptionEnabled = $noSkillProfileOptionEnabled;
    }

    /**
     * @return array
     */
    public function getAvailableSkillProfiles()
    {
        return $this->availableSkillProfiles;
    }

    /**
     * @param array $availableSkillProfiles
     */
    public function setAvailableSkillProfiles($availableSkillProfiles)
    {
        $this->availableSkillProfiles = $availableSkillProfiles;
    }

    /**
     * @return array
     */
    public function getAvailableSkills()
    {
        return $this->availableSkills;
    }

    /**
     * @param array $availableSkills
     */
    public function setAvailableSkills($availableSkills)
    {
        $this->availableSkills = $availableSkills;
    }

    /**
     * @return ilTestSession
     */
    public function getTestSession()
    {
        return $this->testSession;
    }

    protected function init($skillProfileEnabled)
    {
        $testFake = new ilObjTest(0, true);
        $testFake->setTestId($this->testOBJ->getTestId());
        $testFake->setRefId($this->testOBJ->getRefId());
        $testFake->setId($this->testOBJ->getId());
        $testFake->setScoreReporting($this->testOBJ->getScoreReporting());

        $this->testPassesSelector = new ilTestPassesSelector($this->db, $testFake);
        $this->testPassesSelector->setActiveId($this->testSession->getActiveId());
        $this->testPassesSelector->setLastFinishedPass($this->testSession->getPass());

        $assSettings = new ilSetting('assessment');
        $skillEvaluation = new ilTestSkillEvaluation(
            $this->db,
            $this->testOBJ->getTestId(),
            $this->testOBJ->getRefId()
        );

        $skillEvaluation->setUserId($this->getTestSession()->getUserId());
        $skillEvaluation->setActiveId($this->getTestSession()->getActiveId());
        $skillEvaluation->setNumRequiredBookingsForSkillTriggering($assSettings->get(
            'ass_skl_trig_num_answ_barrier',
            ilObjAssessmentFolder::DEFAULT_SKL_TRIG_NUM_ANSWERS_BARRIER
        ));

        $skillEvaluation->init($this->getQuestionList());

        $availableSkillProfiles = $skillEvaluation->getAssignedSkillMatchingSkillProfiles();
        $this->setNoSkillProfileOptionEnabled(
            $skillEvaluation->noProfileMatchingAssignedSkillExists($availableSkillProfiles)
        );
        $this->setAvailableSkillProfiles($availableSkillProfiles);

        $availableSkills = [];

        $skillEvaluation->setPass($this->testSession->getPass());
        $skillEvaluation->evaluate($this->testResults);

        $availableSkills = array_merge($availableSkills, $skillEvaluation->getSkillsMatchingNumAnswersBarrier());

        $this->setAvailableSkills(array_values($availableSkills));
    }

    private function showCmd()
    {
        $selectedSkillProfile = ilTestSkillEvaluationToolbarGUI::fetchSkillProfileParam($_POST);

        $this->init($selectedSkillProfile);

        $evaluationToolbarGUI = $this->buildEvaluationToolbarGUI($selectedSkillProfile);

        $personalSkillsGUI = $this->buildPersonalSkillsGUI(
            $this->getTestSession()->getUserId(),
            $evaluationToolbarGUI->getSelectedEvaluationMode(),
            $this->getAvailableSkills()
        );

        $this->tpl->setContent(
            $this->ctrl->getHTML($evaluationToolbarGUI) . $this->ctrl->getHTML($personalSkillsGUI)
        );
    }

    private function buildEvaluationToolbarGUI($selectedSkillProfileId) : ilTestSkillEvaluationToolbarGUI
    {
        if (!$this->isNoSkillProfileOptionEnabled() && !$selectedSkillProfileId) {
            $selectedSkillProfileId = key($this->getAvailableSkillProfiles());
        }

        $gui = new ilTestSkillEvaluationToolbarGUI($this->ctrl, $this->lng, $this, self::CMD_SHOW);

        $gui->setAvailableSkillProfiles($this->getAvailableSkillProfiles());
        $gui->setNoSkillProfileOptionEnabled($this->isNoSkillProfileOptionEnabled());
        $gui->setSelectedEvaluationMode($selectedSkillProfileId);

        $gui->build();

        return $gui;
    }

    private function buildPersonalSkillsGUI($usrId, $selectedSkillProfileId, $availableSkills) : ilTestPersonalSkillsGUI
    {
        $gui = new ilTestPersonalSkillsGUI($this->lng, $this->testOBJ->getId());

        $gui->setAvailableSkills($availableSkills);
        $gui->setSelectedSkillProfile($selectedSkillProfileId);

        $gui->setUsrId($usrId);

        return $gui;
    }
}
