<?php
/* Copyright (c) 1998-2013 ILIAS open source, Extended GPL, see docs/LICENSE */

/**
 * @ilCtrl_isCalledBy ilObjFormATestGUI: ilRepositoryGUI, ilAdministrationGUI, ilObjPluginDispatchGUI
 * @ilCtrl_Calls      ilObjFormATestGUI: ilPermissionGUI, ilInfoScreenGUI, ilMDEditorGUI, ilObjectCopyGUI, ilPublicUserProfileGUI, ilCommonActionDispatcherGUI
 * @ilCtrl_Calls      ilObjFormATestGUI: ilLearningProgressGUI, ilFormATestLPSummaryTableGUI, ilFormATestLPUsersTableGUI
 * @ilCtrl_Calls      ilObjFormATestGUI: ilAssQuestionPageGUI
 * @ilCtrl_Calls      ilObjFormATestGUI: ilFormATestInfoScreenGUI, ilObjFormATestQuestionSetPoolGUI, ilMarkSchemaGUI
 * @ilCtrl_Calls      ilObjFormATestGUI: ilFormATestSkillAdministrationGUI
 * @ilCtrl_Calls      ilObjFormATestGUI: ilFormATestSkillEvaluationGUI, ilFormATestExportGUI
 */
class ilObjFormATestGUI extends ilObjectPluginGUI implements ilDesktopItemHandling
{
    /**
     * @var ilObjFormATest
     */
    public $object = null;


    /**
     * @var ilPropertyFormGUI
     */
    protected $form;
    
    public $plugin;

    /**
     * @return string
     */
    public function getType()
    {
        return 'xtsf';
    }

    /**
     * @return string
     */
    public function getAfterCreationCmd()
    {
        return 'ilFormATestPropertiesGUI.edit';
    }

    /**
     * @return string
     */
    public function getStandardCmd()
    {
        if ($this->hasPermission('write')) {
            return 'ilFormATestSetsGUI.listSets';
        }

        return 'ilFormATestInfoScreenControllerGUI.show';
    }

    /**
     * Public wrapper for permission checks
     * @param string $permission
     * @return bool
     */
    public function hasPermission($permission)
    {
        return $this->checkPermissionBool($permission);
    }

    /**
     * Public wrapper for permission assumption
     * @param string $permission
     * @return bool
     */
    public function ensurePermission($permission)
    {
        return $this->checkPermission($permission);
    }

    /**
     * @param string $cmd
     */
    public function performCommand($cmd)
    {
        global $DIC;
        $ilTabs = $DIC->tabs();
        $tpl = $DIC->ui()->mainTemplate();
        $ilDB = $DIC->database();

        $this->lng->loadLanguageModule('assessment');
        $tpl->addCss('./Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/templates/default/xtsf.css');

        $tpl->setDescription($this->object->getDescription());

        $next_class = $this->ctrl->getNextClass($this);
        switch ($next_class) {

            case 'ilmarkschemagui':
                $this->checkPermission('write');
                $this->setSubTabs($ilTabs, 'ilmarkschemagui');
                $mark_schema_gui = new ilMarkSchemaGUI($this->object);
                $this->ctrl->forwardCommand($mark_schema_gui);
                $ilTabs->activateTab('properties'); // Has to be called at last, because ilMarkSchemaGUI sets the active tab to `settings`
                break;
            case 'ilformatestskilladministrationgui':
                $this->prepareOutput();
                $this->addHeaderAction();

                $gui = new ilFormATestSkillAdministrationGUI($this);
                $this->ctrl->forwardCommand($gui);
                break;
            case 'ilformatestskillevaluationgui':
                $this->prepareOutput();
                $this->addHeaderAction();

                $session = new ilFormATestSession($this->object);
                $session->read($this->object->getSetList()->getData());

                $helper = new ilFormATestSkillLevelHelper();
                $helper->setSession($session);

                $questionList = $helper->loadQuestionList($this->object);
                $testResults = $helper->getTestResults($this->object);

                $objectiveOrientedContainer = new ilTestObjectiveOrientedContainer();
                $objectiveOrientedContainer->setObjId($this->object->getId());
                $objectiveOrientedContainer->setRefId($this->object->getRefId());

                $gui = new ilFormATestSkillEvaluationGUI(
                    $this->ctrl,
                    $ilTabs,
                    $this->tpl,
                    $this->lng,
                    $ilDB,
                    $this->object
                );
                $gui->setQuestionList($questionList);
                $gui->setTestSession($session);
                $gui->setTestResults($testResults);
                $gui->setObjectiveOrientedContainer($objectiveOrientedContainer);
                $this->ctrl->forwardCommand($gui);
                $ilTabs->clearTargets();
                $ilTabs->setBackTarget(
                    $this->lng->txt('tst_results_back_introduction'),
                    $this->ctrl->getLinkTarget($this, 'ilFormATestInfoScreenControllerGUI.show')
               );
                break;

            case 'ilmdeditorgui':
                $this->checkPermission('write');
                $ilTabs->activateTab('meta_data');
                $md_gui = new ilMDEditorGUI($this->object->getId(), 0, $this->object->getType());
                $md_gui->addObserver($this->object, 'MDUpdateListener', 'General');
                $ilTabs->activateSubTab('meta_data');
                $this->ctrl->forwardCommand($md_gui);
                break;

            case 'ilpublicuserprofilegui':
                $profile_gui = new ilPublicUserProfileGUI($_GET['user']);
                $profile_gui->setBackUrl($this->ctrl->getLinkTarget($this, 'ilFormATestInfoScreenControllerGUI.show'));
                $this->tpl->setContent($this->ctrl->forwardCommand($profile_gui));
                break;
            case "ilassquestionpagegui":
                $gui = new ilAssQuestionPageGUI();
                $this->tpl->setContent($this->ctrl->forwardCommand($gui));
                break;
            case 'ilcommonactiondispatchergui':
                $gui = ilCommonActionDispatcherGUI::getInstanceFromAjaxCall();
                $this->ctrl->forwardCommand($gui);
                break;
            case 'ilobjformatestquestionsetpoolgui':
                $gui = new ilObjFormATestQuestionSetPoolGUI((int) $_GET['ref_id']);
                $this->ctrl->forwardCommand($gui);
                break;
            case "ilformatestexportgui":
                $this->checkPermission('write');
                $ilTabs->setTabActive('export');
                $this->plugin->includeClass('class.ilFormATestExportGUI.php');
                $exp_gui = new ilFormATestExportGUI($this);
                $exp_gui->addFormat('xml', $this->lng->txt('qpl_export_xml'));
                $this->ctrl->forwardCommand($exp_gui);
                break;
            default:
                switch ($cmd) {
                    case 'updateProperties':
                    case 'editProperties':
                    case 'showHistory':
                        $this->checkPermission('write');
                        $this->$cmd();
                        break;

                    case 'addToDesk':
                    case 'removeFromDesk':
                    case 'redrawHeaderAction':
                        if (in_array($cmd, array('addToDesk', 'removeFromDesk'))) {
                            $cmd .= 'Object';
                        }
                        $this->checkPermission('read');
                        $this->$cmd();
                        break;
                    case "handleQuestionAction":
                    case "showInstantResponse":
                        $cmd = "ilFormATestPlayerControllerGUI." . $cmd;
                        // no break
                    default:
                        $dispatcher = ilXTSFPluginDispatcher::getInstance($this);
                        $this->setSubTabs($ilTabs, $dispatcher->getController($cmd));
                        $dispatcher->dispatch($cmd);
                        break;
                }
                break;
        }

        if (strpos($cmd, 'ilFormATestPlayerControllerGUI') === false) {
            $this->addHeaderAction();
        }
    }

    /**
     * Copied from: ilInfoScreenGUI
     */
    public function saveTags()
    {
        $tagging_gui = new ilTaggingGUI();
        $tagging_gui->setObject($this->object->getId(), $this->object->getType());
        $tagging_gui->saveInput();

        $this->ctrl->redirect($this, 'ilFormATestInfoScreenControllerGUI.show');
    }

    /**
     *
     */
    protected function setTabs()
    {
        global $DIC;
        $ilTabs = $DIC->tabs();
        $ilCtrl = $DIC->ctrl();
        $lng = $DIC->language();

        $ilTabs->clearTargets();

        if ($this->checkPermissionBool('write')) {
            $ilTabs->addTab('sets', $this->txt('sets'), $ilCtrl->getLinkTarget($this, 'ilFormATestSetsGUI.listSets'));
        }

        if ($this->checkPermissionBool("read")) {
            $ilTabs->addTab("info_short", $lng->txt("info_short"), $ilCtrl->getLinkTarget($this, "ilFormATestInfoScreenControllerGUI.show"));
        }

        //		$this->addInfoTab();

        if ($this->checkPermissionBool('write')) {
            $ilTabs->addTab('properties', $this->txt('properties'), $ilCtrl->getLinkTarget($this, 'ilFormATestPropertiesGUI.edit'));
        }

        if ($this->checkPermissionBool('read') && $this->object->isSkillServiceEnabled() && ilObjFormATest::isSkillManagementGloballyActivated()) {
            $link = $this->ctrl->getLinkTargetByClass(
                array('ilFormATestSkillAdministrationGUI', 'ilFormATestAssQuestionSkillAssignmentsGUI'),
                ilFormATestAssQuestionSkillAssignmentsGUI::CMD_SHOW_SKILL_QUEST_ASSIGNS
            );

            $ilTabs->addTarget('tst_tab_competences', $link);
        }

        if ($this->checkPermissionBool('write')) {
            $ilTabs->addTab('participants', $this->txt('participants'), $ilCtrl->getLinkTarget($this, 'ilFormATestParticipantsGUI.show'));
        }

        if (ilLearningProgressAccess::checkAccess($this->object->getRefId(), true)) {
            if ($this->checkPermissionBool('write')) {
                if ($this->object->getLearningProgressMode() != ilObjFormATest::LP_MODE_DEACTIVATED) {
                    $ilTabs->addTab('learning_progress', $lng->txt('learning_progress'), $ilCtrl->getLinkTarget($this, 'ilFormATestLearningProgressGUI.showLpUsers'));
                } else {
                    $ilTabs->addTab('learning_progress', $lng->txt('learning_progress'), $ilCtrl->getLinkTarget($this, 'ilFormATestLearningProgressGUI.showLPSettings'));
                }
            } elseif ($this->checkPermissionBool('read') && $this->object->getLearningProgressMode() != ilObjFormATest::LP_MODE_DEACTIVATED) {
                $ilTabs->addTab('learning_progress', $lng->txt('learning_progress'), $ilCtrl->getLinkTarget($this, 'ilFormATestLearningProgressGUI.showLPUserDetails'));
            }
        }

        if ($this->checkPermissionBool('write')) {
            $ilTabs->addTab('history', $lng->txt('history'), $ilCtrl->getLinkTarget($this, 'ilFormATestHistoryGUI.showHistory'));
            $ilTabs->addTarget('meta_data', $this->ctrl->getLinkTargetByClass('ilmdeditorgui', ''), '', 'ilmdeditorgui');
            $ilTabs->addTarget("export", $this->ctrl->getLinkTargetByClass("ilformatestexportgui", ""), "", "ilformatestexportgui");
        }

        $this->addPermissionTab();
    }

    private function setSubTabs(ilTabsGUI $tabsGUI, $controllerClass)
    {
        switch ($controllerClass) {
            case 'ilFormATestPropertiesGUI':
            case 'ilFormATestScoringAndResultPropertiesGUI':
            case 'ilmarkschemagui':

                $this->setPropertiesSubTabs($tabsGUI, $controllerClass);
                break;
        }
    }

    private function setPropertiesSubTabs(ilTabsGUI $tabsGUI, $controllerClass)
    {
        $tabsGUI->addSubTab(
            'xtsf_properties',
            $this->txt('st_general_properties'),
            $this->ctrl->getLinkTarget($this, 'ilFormATestPropertiesGUI.edit')
        );

        $tabsGUI->addSubTab(
            'xtsf_markschema',
            $this->txt('st_mark_schema'),
            $this->ctrl->getLinkTargetByClass('ilMarkSchemaGUI', 'showMarkSchema')
        );
        
        $tabsGUI->addSubTab(
            'xtsf_scoringresult',
            $this->txt('st_scoring_result'),
            $this->ctrl->getLinkTarget($this, 'ilFormATestScoringAndResultPropertiesGUI.edit')
        );

        switch ($controllerClass) {
            case 'ilFormATestScoringAndResultPropertiesGUI':

                $tabsGUI->activateSubTab('xtsf_scoringresult');
                break;

            case 'ilFormATestPropertiesGUI':

                $tabsGUI->activateSubTab('xtsf_properties');
                break;

            case 'ilmarkschemagui':

                $tabsGUI->activateSubTab('xtsf_markschema');
                break;
        }
    }

    /**
     *
     */
    public function getObjId()
    {
        return $this->object->getId();
    }

    /**
     * @param string $a_sub_type
     * @param int $a_sub_id
     * @return ilObjectListGUI|ilObjFormATestListGUI
     */
    protected function initHeaderAction($a_sub_type = null, $a_sub_id = null)
    {
        global $DIC;
        $ilUser = $DIC->user();

        $lg = parent::initHeaderAction();
        if ($lg instanceof ilObjFormATestListGUI) {
            if ($ilUser->getId() != ANONYMOUS_USER_ID) {
                // Maybe handle notifications in future ...
            }
        }

        return $lg;
    }

    /**
     * @param string $type
     * @return array
     */
    protected function initCreationForms($type)
    {
        return array(
            self::CFORM_NEW => $this->initCreateForm($type),
            self::CFORM_IMPORT => $this->initImportForm($type)
        );
    }

    /**
     * @param string $type
     * @return ilPropertyFormGUI
     */
    public function initCreateForm($type)
    {
        $form = parent::initCreateForm($type);

        // Add additional form elements for creation form (e.g. clone, import)

        return $form;
    }
    
    /**
     * @return array
     */
    private function getXqspFormOptionValues()
    {
        $xqsp_available_pools = ilObjQuestionSetPool::getAvailableQuestionSetPools(true);
        $options_1 = array(ilFormATestImporter::SINGLE_IMPORT_NEW_POOL => $this->plugin->txt("new_xqsp_pool"));
        
        if (is_array($xqsp_available_pools) && count($xqsp_available_pools)) {
            foreach ($xqsp_available_pools as $key => $xqsp_obj) {
                $options_1[$key] = $xqsp_obj->getTitle();
            }
        }
        return $options_1;
    }
    
    /**
     * @return array
     */
    private function getQplFormOptionValues()
    {
        $tst = new ilObjTest();
        $questionpools = $tst->getAvailableQuestionpools(true, false, true, true);
        $this->plugin->includeClass('class.ilFormATestImporter.php');
        $options = array(
            ilFormATestImporter::SINGLE_IMPORT_NEW_POOL => $this->lng->txt("assessment_new_pool"),
            ilFormATestImporter::SINGLE_IMPORT_NO_POOL => $this->lng->txt('assessment_no_pool')
        );
        
        if (is_array($questionpools) && count($questionpools)) {
            foreach ($questionpools as $key => $value) {
                $options[$key] = $value["title"];
            }
        }
        return $options;
    }
    
    /**
     * @param $a_new_type
     * @return ilPropertyFormGUI
     */
    public function initImportForm($a_new_type)
    {
        $form = parent::initImportForm($a_new_type);
        $this->plugin->includeClass('class.ilFormATestImporter.php');
        
        // select QuestionSetPool
        $radio_grp = new ilRadioGroupInputGUI($this->plugin->txt('xqsp_selection'), 'xqsp_selection');
        
        // no QuestionSetPool
        $no_xqsp = new ilRadioOption($this->plugin->txt('no_xqsp_pool'), 'xqsp_sel_0', $this->plugin->txt('no_xqsp_pool_info'));
        
        $radio_grp->addOption($no_xqsp);
        
        // new/existing QuestionSetPool
        $xqsp_pool = new ilSelectInputGUI($this->plugin->txt("select_xqsp_pool"), "xqsp");
        $xqsp_pool->setOptions($this->getXqspFormOptionValues());
        
        // select question pool
        $qpl_pool = new ilSelectInputGUI($this->lng->txt("select_questionpool"), "qpl");
        $qpl_pool->setOptions($this->getQplFormOptionValues());
        
        $existing_xqsp = new ilRadioOption($this->plugin->txt('existing_xqsp_pool'), 'xqsp_sel_1', $this->plugin->txt('existing_xqsp_pool_info'));
        $existing_xqsp->addSubItem($xqsp_pool);
        $existing_xqsp->addSubItem($qpl_pool);
        
        $radio_grp->setValue('xqsp_sel_0');
        $radio_grp->addOption($existing_xqsp);
        $form->addItem($radio_grp);
        
        return $form;
    }
    
    /**
     *
     */
    protected function editProperties()
    {
        global $DIC;

        $this->initPropertiesForm();
        $this->getPropertiesValues();

        $DIC->ui()->mainTemplate()->setContent($this->form->getHTML());
    }

    /**
     *
     */
    protected function initPropertiesForm()
    {
        global $DIC;
        $ilCtrl = $DIC->ctrl();

        $this->form = new ilPropertyFormGUI();
        $this->form->setTitle($this->txt('edit_properties'));
        $this->form->setFormAction($ilCtrl->getFormAction($this, 'updateProperties'));

        $ti = new ilTextInputGUI($this->txt('title'), 'title');
        $ti->setRequired(true);
        $this->form->addItem($ti);

        $ta = new ilTextAreaInputGUI($this->txt('description'), 'desc');
        $this->form->addItem($ta);

        // Add your form elements here

        $this->form->addCommandButton('updateProperties', $this->txt('save'));
    }

    /**
     *
     */
    protected function getPropertiesValues()
    {
        $values['title'] = $this->object->getTitle();
        $values['desc'] = $this->object->getDescription();

        // Add other settings here

        $this->form->setValuesByArray($values);
    }

    /**
     *
     */
    protected function updateProperties()
    {
        global $DIC;
        $tpl = $DIC->ui()->mainTemplate();
        $lng = $DIC->language();
        $ilCtrl = $DIC->ctrl();

        $this->initPropertiesForm();
        if ($this->form->checkInput()) {
            $this->object->setTitle($this->form->getInput('title'));
            $this->object->setDescription($this->form->getInput('desc'));

            // Set settings from request to object here

            $this->object->update();
            ilUtil::sendSuccess($lng->txt('msg_obj_modified'), true);
            $ilCtrl->redirect($this, 'editProperties');
        }

        $this->form->setValuesByPost();
        $tpl->setContent($this->form->getHTML());
    }

    /**
     * Overwriting this method is necessary to handle creation problems with the api
     */
    public function save()
    {
        $this->saveObject();
    }

    /**
     * Overwriting this method is necessary to handle creation problems with the api
     */
    public function saveObject()
    {
        global $DIC;
        $ilCtrl = $DIC->ctrl();

        try {
            parent::saveObject();
        } catch (Exception $e) {
            ilUtil::sendFailure($this->plugin->txt($e->getMessage()), true);
            $ilCtrl->setParameterByClass('ilrepositorygui', 'ref_id', (int) $_GET['ref_id']);
            $ilCtrl->redirectByClass('ilrepositorygui');
        }
    }

    /**
     * @param ilObjFormATest $newObj
     */
    public function afterSave(ilObject $newObj)
    {
        // Do something on save-request, e.g. add members to your object automatically etc.
        parent::afterSave($newObj);
    }

    public static function _goto($a_target, $a_thread = 0, $a_posting = 0)
    {
        global $DIC;
        $ilCtrl = $DIC->ctrl();
        $ilAccess = $DIC->access();
        $lng = $DIC->language();

        $t = explode("_", $a_target[0]);
        $ref_id = (int) $t[0];
        $class_name = $a_target[1];

        if ($ilAccess->checkAccess("write", "", $ref_id)) {
            $ilCtrl->initBaseClass("ilObjPluginDispatchGUI");
            $ilCtrl->setTargetScript("ilias.php");
            $ilCtrl->getCallStructure(strtolower("ilObjPluginDispatchGUI"));
            $ilCtrl->setParameterByClass($class_name, "ref_id", $ref_id);
            $ilCtrl->redirectByClass(array("ilobjplugindispatchgui", $class_name), "ilFormATestSetsGUI.listSets");
        } elseif ($ilAccess->checkAccess("read", "", $ref_id)) {
            $ilCtrl->initBaseClass("ilObjPluginDispatchGUI");
            $ilCtrl->setTargetScript("ilias.php");
            $ilCtrl->getCallStructure(strtolower("ilObjPluginDispatchGUI"));
            $ilCtrl->setParameterByClass($class_name, "ref_id", $ref_id);
            $ilCtrl->redirectByClass(array("ilobjplugindispatchgui", $class_name), "ilFormATestInfoScreenControllerGUI.show");
        } elseif ($ilAccess->checkAccess("visible", "", $ref_id)) {
            $ilCtrl->initBaseClass("ilObjPluginDispatchGUI");
            $ilCtrl->setTargetScript("ilias.php");
            $ilCtrl->getCallStructure(strtolower("ilObjPluginDispatchGUI"));
            $ilCtrl->setParameterByClass($class_name, "ref_id", $ref_id);
            $ilCtrl->redirectByClass(array("ilobjplugindispatchgui", $class_name), "infoScreen");
        } elseif ($ilAccess->checkAccess("read", "", ROOT_FOLDER_ID)) {
            ilUtil::sendFailure(sprintf(
                $lng->txt("msg_no_perm_read_item"),
                ilObject::_lookupTitle(ilObject::_lookupObjId($ref_id))
            ));
            ilObjectGUI::_gotoRepositoryRoot();
        }
    }
}
