<?php

/**
 * Class ilObjFormATestQuestionSetPoolGUI
 *
 * Date: 14.02.14
 * Time: 14:01
 * @author Thomas Joußen <tjoussen@databay.de>
 */
class ilObjFormATestQuestionSetPoolGUI extends ilObjQuestionSetPoolGUI
{

    /**
     * execute command
     */
    public function &executeCommand()
    {
        global $DIC;
        $ilCtrl = $DIC->ctrl();
        $tpl = $DIC->ui()->mainTemplate();
        $lng = $DIC->language();
        $ilTabs = $DIC->tabs();
        $ilLocator = $DIC['ilLocator'];
        $ilToolbar = $DIC->toolbar();

        $available_pools = array_keys(ilObjQuestionSetPool::getAvailableQuestionSetPools());

        $set = new ilQuestionSetPoolSet((int) $_GET["set_id"]);
        $set->read();

        // get standard template (includes main menu and general layout)
        $tpl->loadStandardTemplate();

        $ilLocator->clearItems();

        $plugin = $this->initFormATestPlugin();

        $tpl->setTitle($this->object->getTitle());
        $tpl->setTitleIcon(
            $plugin->getImagePath("icon_" . $this->object->getType() . ".svg"),
            $lng->txt("icon") . " " . $this->txt("obj_" . $this->object->getType())
        );

        $this->setTabs();
        $this->setLocator();

        $next_class = $this->ctrl->getNextClass($this);
        $cmd = $this->ctrl->getCmd();

        switch ($next_class) {
            case "ilinfoscreengui":
                $this->checkPermission("visible");
                $this->infoScreen();	// forwards command
                break;

            case 'ilpermissiongui':
                $perm_gui = new ilPermissionGUI($this);
                $ilTabs->setTabActive("perm_settings");
                $ret = $ilCtrl->forwardCommand($perm_gui);
                break;

            case 'ilobjectcopygui':
                $cp = new ilObjectCopyGUI($this);
                $cp->setType($this->getType());
                $this->ctrl->forwardCommand($cp);
                break;

            default:
                if (strtolower($_GET["baseClass"]) == "iladministrationgui") {
                    $this->viewObject();
                    return;
                }
                if (!$cmd) {
                    $cmd = $this->getStandardCmd();
                }
                if ($cmd == "infoScreen") {
                    $ilCtrl->setCmd("showSummary");
                    $ilCtrl->setCmdClass("ilinfoscreengui");
                    $this->infoScreen();
                } else {
                    $this->performCommand($cmd);

                    $reflection = new ReflectionObject($this->tpl);
                    $contentTemplateProp = $reflection->getProperty('legacy_content_template');
                    $contentTemplateProp->setAccessible(true);
                    $contentTemplate = $contentTemplateProp->getValue($this->tpl);

                    $contentTemplateReflection = new ReflectionObject($contentTemplate);
                    $property = $contentTemplateReflection->getProperty('message');
                    $property->setAccessible(true);
                    $messages = $property->getValue($contentTemplate);
                    $messages = (array) $messages;

                    $hasFailure = (
                        (
                            isset($messages['failure'])
                            && is_string($messages['failure']) &&
                            0 !== strlen($messages['failure'])
                        ) ||
                        (
                            is_string(ilSession::get('failure')) &&
                            0 !== ilSession::get('failure')
                        )
                    );

                    if (ilObjQuestionSetPool::canSynchronizeSet($set->getOriginalId())) {
                        if (!$hasFailure) {
                            $ilCtrl->setParameter($this, "set_id", $set->getId());
                            $ilToolbar->setFormAction($ilCtrl->getFormAction($this));
                            $ilCtrl->setParameter($this, "set_id", "");
                            $ilToolbar->addFormButton($this->plugin->txt("synchronize_set"), "ilQuestionSetPoolSetsGUI.synchronize", "", true);
                        }
                    } elseif (!$hasFailure) {
                        ilUtil::sendInfo($this->plugin->txt("not_in_test"));
                    }
                }
                break;
        }
    }

    protected function redirectToDefaultPage()
    {
        $this->ctrl->redirectByClass("ilObjFormATestGUI", "ilFormATestSetsGUI.listSets");
    }

    public function initFormATestPlugin()
    {
        return ilPlugin::getPluginObject(
            IL_COMP_SERVICE,
            "Repository",
            "robj",
            ilPlugin::lookupNameForId(IL_COMP_SERVICE, "Repository", "robj", "xtsf")
        );
    }

    public function setTabs()
    {
        global $DIC;
        $ilTabs = $DIC->tabs();

        $ilTabs->clearTargets();

        $ilTabs->setBackTarget($this->txt('back_to_test'), $this->ctrl->getLinkTargetByClass("ilObjFormATestGUI", "ilFormATestSetsGUI.listSets"));
    }

    protected function addHeaderAction()
    {
    }

    /**
     * Add object to locator
     */
    public function addLocatorItems()
    {
        global $DIC;
        $ilLocator = $DIC['ilLocator'];

        if (!$this->getCreationMode()) {
            $ilLocator->addItem(
                $this->object->getTitle(),
                $this->ctrl->getLinkTargetByClass('ilObjFormATestGUI', $this->getStandardCmd()),
                "",
                $_GET["ref_id"]
            );
        }
    }

    public function getStandardCmd()
    {
        return "ilFormATestSetsGUI.listSets";
    }
}
